/*************************************************************************************
__MXSDOC__
Author:				Gavyn Thompson
Company:				GTVFX
Website:				www.gtvfx.com
Email:				gthompson@gtvfx.com
ScriptVersion:			v1.05
Updated:				12/31/2013
[Purpose]
Install for main script library MZP file.
__END__
*************************************************************************************/
struct toolBoxInstall_lib
(
	self,
	ro,
	tempDir = ((getDir #temp)+@"\GTVFX"),
	scriptsDir = ((getDir #scripts)+@"\GTVFX\"),
	startupDir = ((getDir #userStartupScripts)+"\\"),
	macrosDir = ((getDir #userMacros)+"\\"),
	tempGTVFX = (tempDir + @"\Scripts\GTVFX"),
	tempStartup = (tempDir + @"\Scripts\Startup"),
	tempMacros = (tempDir + @"\UI\usermacros"),
	
	fn removeTempDir =
	(
		if doesFileExist tempDir then dosCommand ("rmdir \"" + tempDir + "\" /s /q")
	),
	fn copyFiles_FN fromDir toDir =
	(
		if not doesFileExist fromDir then
		(
			return false
		)
		else
		(		
			sFileArr = getFiles (fromDir + @"\*")
			for i in sFileArr do copyFile i (toDir+(getFileNameFile i)+(getFileNameType i))
		)
		true
	),
	fn initCallBacks =
	(
		sFile = (getFiles ((getDir #scripts) + @"\GTVFX\*GTVFX-toolbox_callbacksLib*.*"))[1]
		if doesFileExist sFile then filein sFile else return false
		true
	),
	fn initMenu =
	(
		sFile = (getFiles ((getDir #scripts) + @"\GTVFX\*GTVFX-toolbox_MacrosLib*.*"))[1]
		if doesFileExist sFile then filein sFile else return false
		true
	),
	fn install_FN =
	(
		if not (makeDir scriptsDir) then return false
		if not copyFiles_FN tempStartup startupDir then return false
		if not copyFiles_FN tempGTVFX scriptsDir then return false
		true
	),
	fn uninstall_FN arr:#() =
	(
		if doesFileExist startupDir then arr += getFiles (startupDir +"GTVFX*")
		if doesFileExist macrosDir then arr += getFiles (macrosDir +"GTVFX*")
		if arr.count != 0 then for i in arr do deleteFile i
		if doesFileExist scriptsDir then dosCommand ("rmdir \"" + scriptsDir + "\" /s /q")
		try(layIso.purgeLayIsoItems())catch()
	),
	fn launchToolbox_FN =
	(
		sFile = (getFiles((getDir #scripts) + @"\GTVFX\GTVFX-toolbox_Loader.*"))[1]
		if doesFileExist sFile then filein sFile else(messageBox "Cannot find toolbox script file")
	),
	fn unregisterMenue =
    (
        maxMenu = menuMan.getMainMenuBar()
        GTVFX_toolboxMenu = menuMan.findMenu "GTVFX:Toolbox"
        try(menuMan.unRegisterMenu GTVFX_toolboxMenu)catch()
        menuMan.updateMenuBar()
    ),
	fn ui =
	(
		rollout ro "GTVFX: Install" width:250
		(
			local self
			local dnToolTip
			local clrWindow = ((colorMan.getColor #window)*255)
			local clrText = ((colorMan.getColor #text)*255)
			local ClrBackGround = ((colorMan.getColor #background)*255)
			dotNetControl dNbtn_install "button" width:(ro.width-25) height:40
			dotNetControl dNbtn_unInstall "button" width:(ro.width-25) height:30
			hyperLink hyp_website "www.gtvfx.com" color:orange  hoverColor:red visitedColor:orange address:"http://www.gtvfx.com"
			fn initToolTip dNetObj caption =
			(
				if dnTooltip == undefined then
				(
					dnToolTip = dotnetobject "ToolTip"
					dnToolTip.AutoPopDelay = 5000
					dnToolTip.InitialDelay = 300
					dnToolTip.ReshowDelay = 300
					dnToolTip.ShowAlways = true
					dnToolTip.IsBalloon = true
				)
				dnToolTip.SetToolTip dNetObj caption
			)
			fn normalizeRGB val =
			(
				if val <0 then val = 0 else if val >255 then val = 255
				val
			)
			fn setDotNetWidget dNobj caption fontSize colorOffsetInt:0 =
			(
				dNobj.text = caption
				dNobj.forecolor = dNobj.forecolor.FromArgb clrText.x clrText.y clrText.z
				dNobj.backColor = dNobj.backcolor.FromArgb (normalizeRGB (ClrBackGround.x+colorOffsetInt)) (normalizeRGB (ClrBackGround.y+colorOffsetInt)) (normalizeRGB (ClrBackGround.z+colorOffsetInt))
				dNobj.Font = dotNetObject "System.Drawing.Font" "Tahoma" fontSize ((dotNetClass "System.Drawing.FontStyle").bold)
			)
			fn initDnetBtn dNbtn caption fontSize style:#popup colorOffsetInt:0 tooltip:"" = 
			(
				case style of
				(
					#flat:(dNbtn.flatStyle = dNbtn.flatStyle.flat)
					#popup:(dNbtn.flatStyle = dNbtn.flatStyle.popup)
					#system:(dNbtn.flatStyle = dNbtn.flatStyle.system)
				)
				setDotNetWidget dNbtn caption fontSize colorOffsetInt:colorOffsetInt
				initToolTip dNbtn tooltip
				dNbtn.tag = tooltip
			)
			fn posHyperLink posX posY =
			(
				hyp_website.pos = [posX,posY]
			)
			fn _init pself=
			(
				self = pself
				initDnetBtn dNbtn_install "Install Script Library" 10 style:#popup colorOffsetInt:10
				initDnetBtn dNbtn_unInstall "Uninstall Script Library" 9 style:#popup colorOffsetInt:-5
			)
			on dNbtn_install mouseClick do
			(
				self.uninstall_FN()
				if self.install_FN() then
				(
					self.initCallBacks()
					self.launchToolbox_FN()
					self.initMenu()
					self.removeTempDir()
					messageBox "GTVFX Toolbox library successfully installed.\nYou may need to restart Max if you are updating to a newer version of the Toolbox." title:"GTVFX: Success"
				)
				else messageBox "Could not install the tool.\nDouble check that you have permissions to copy files to the 3ds Max root directory." title:"GTVFX: Error"
				try(destroyDialog self.ro)catch() 
			)
			on dNbtn_unInstall mouseClick do
			(
				if queryBox "Are you sure you want to remove the script library? :(" then
				(
					self.uninstall_FN()
					try(closeRolloutFloater gtb.gtbFloater)catch()
					try(self.unregisterMenue())catch()
					try(destroyDialog self.ro)catch()
					messageBox "Script library removed." title:"GTVFX: :("
				)
			)
			on ro open do
			(
				print ro.height
				posHyperLink (ro.width/2-40) (ro.height - 20)
			)
		)
		createDialog ro
	),
	fn _init =
	(
		self = this
		ui()
		ro._init self
	),
	init = _init()
)
gtvfx = toolBoxInstall_lib()